// Read values that are transmitted on the serial port 
// and push them to a Pachube datastream

import eeml.*;
import processing.serial.*;

// program constants

// ASCII linefeed is the delimitor sent by the transmitting side to
// indicate the end of a one data transmission
int    lineFeed = 10;      // ASCII linefeed

// The key is needed for identification purposes
// If the feed you read is public, any key will do
// if the feed is private, you'll need the specific key of the feed owner
String apiKey   = "IbMuMnl9sEhB62GlKNqzT88z8smSAKxwRXVPdk1oQ2o2az0g";

// The url to use to read the feed, usually something like 
// http://www.pachube.com/api/xxxxx.xml
// where the xxxxx is the feed ID that was generated when creating the feed
// (also visible when selecting the feed on the Pachube website
String feedURL  = "http://www.pachube.com/api/91613.xml";

// Serial port used for communication
String portName = "COM16";


DataOut dOut;                // Declare the Pachube outputstream object
Serial  myPort;              // Declare the Serial port object
boolean dataToggle = false;  // Declare the boolean that tracks data activity
int     bgColor = 0;         // Declare the variable that holds the current background color

void setup() {
  // set up the serial port, check the definition of port name above!!
  myPort = new Serial(this, portName, 38400);
  myPort.bufferUntil(lineFeed); // make serialevent function trigger on linefeeds...

  // set up DataOut object; requires URL of the EEML you are updating, 
  // and your Pachube API key   
  dOut = new DataOut(this, feedURL, apiKey);
  
  //  and add and tag a datastream    
  dOut.addData(0, "Ldrval");

  // set color for activity dot
  noStroke();
  fill(255, 0, 0);
}

void draw()
{
  // redraw background in current color
  background(bgColor);
  // signal incoming data with flashing dot
  if (dataToggle)
    ellipse(5,5,10,10);
}

// We assume every now and then a line containing a single int
// value is received, ended by a newline, e.g. sent by an arduino 
// with some analog sensor
void serialEvent(Serial myPort) {
  // here the "readable text received is converted to an actual value
  // first strip excess spaces, linefeeds, etc.
  String instring = trim(myPort.readString());
  // convert the text string to an int value (we assume tha this is possible
  // so no invalid characters should be in the string, only '0'..'9')
  int inval = int(instring);
  // map the range of the sensor value to 0..255. The 600/1023 values 
  // used here should be replaces by values dependent on YOUR sensor 
  // range.
  bgColor = int(255-map(inval, 600, 1023, 0, 255));
  // indicate data is received
  dataToggle = ! dataToggle;
  // send either a 0 or 1 to the serial port depending on the dataToggle value
  myPort.write((dataToggle?1:0));
  // prepare the outputstream for update by storing the value read
  dOut.update(0, inval);
  // updatePachube() updates by an authenticated PUT HTTP request
  int response = dOut.updatePachube();
  // the response should be 200 if successful; 
  // 401 if unauthorized; 404 if feed doesn't exist; ...
  if (response != 200) {
    println("response = " + response);
  }  
}




